<?php namespace EwTools\Admin\ProjectPluginPage;

use EwTools\Interfaces\PluginPageInterface;
use EwTools\Interfaces\SettingsPageInterface;
use EwTools\Support\Collection;
use EwTools\Support\Str;
use EwTools\Twig\View;

/**
 * SettingsPage
 *
 * @link    http://infomaker.se
 * @package EwTools\Admin\ProjectPluginPage
 * @since   EwTools\Admin\ProjectPluginPage\SettingsPage 0.1
 */
class SettingsPage implements PluginPageInterface {
    
    /**
     * @var string
     */
    protected static $slug = 'settings';
    
    /**
     * @var string
     */
    protected static $title = 'Settings';
    
    /**
     * @var string
     */
    protected $text_domain = 'ewtools_textdomain';
    
    /**
     * Contains messages from the tab-pages
     *
     * @var array
     */
    protected $messages;
    
    /**
     * @var string
     */
    private $parent_slug;
    
    /**
     * Contains the pages added as tab to the plugins settings-page
     *
     * @var SettingsPageInterface[]
     */
    private $settings_pages = [];
    
    /**
     * Contains the settings-page currently active
     *
     * @var SettingsPageInterface
     */
    private $current_page = null;
    
    /**
     * SettingsPage constructor.
     *
     * @since 0.1
     *
     * @param string $parent_slug
     */
    public function __construct( $parent_slug ) {
        
        $this->parent_slug = Str::slug( $parent_slug );
    }
    
    /**
     * Add a page to be displayed as a tab on the settings page
     *
     * @param SettingsPageInterface $page
     *
     * @since 0.1
     * @return $this
     */
    public function addSettingsTab( SettingsPageInterface $page ) {
        $this->settings_pages[ $page->getTabSlug() ] = $page;
        
        return $this;
    }
    
    /**
     * Use {add_submenu_page} to add the page to the plugin menu.
     *
     * @param string $parent_slug
     *
     * @since 0.1
     * @return string
     */
    public function addToMenu( $parent_slug ) {
        $hook = add_submenu_page(
            $parent_slug,
            __( $this->getTitle(), $this->text_domain ),
            __( $this->getTitle(), $this->text_domain ),
            'manage_options',
            $parent_slug, [ $this, 'renderSettingsPage' ]
        );
        
        add_action( "load-{$hook}", [ $this, 'onSettingsPageLoad' ] );
    }
    
    /**
     * Determine and fetch the current page based on the active tab
     *
     * @since 0.1
     * @return mixed
     */
    protected function getCurrentPage() {
        return $this->getSettingsTabs()->get( $this->getActiveTab() );
    }
    
    /**
     * Retrieve the slug to be use to identify the page
     *
     * @since 0.1
     * @return mixed
     */
    public function getSlug() {
        return Str::slug( static::$slug );
    }
    
    /**
     * Retrieve the title of the page to be used in the menu
     *
     * @since 0.1
     * @return string
     */
    public function getTitle() {
        return Str::title( static::$title );
    }
    
    /**
     * Fires whenever the settings page is loaded
     *
     * @since 0.1
     * @return void
     */
    public function onSettingsPageLoad() {
        $page = $this->getCurrentPage();
        if( $page instanceof SettingsPageInterface ) {
            $page->onPageLoad();
            $this->current_page = $page;
        }
    }
    
    /**
     * Display settings page
     */
    public function renderSettingsPage() {
        View::render( '@tools/admin/plugin-settings-page', [
            'tabs'         => $this->getTabs(),
            'text_domain'  => $this->text_domain,
            'message'     => $this->getMessage(),
            'page_content' => $this->current_page->pageContent()
        ] );
    }
    
    /**
     * Collect data from each registered tab-page
     *
     * @since 0.1
     * @return array
     */
    protected function getTabs() {
        $active_tab = $this->getActiveTab();
        
        return array_map( function ( $page ) use ( $active_tab ) {
            return [
                'slug'   => $page->getTabSlug(),
                'label'  => $page->getTabTitle(),
                'active' => $page->getTabSlug() === $active_tab,
                'url'    => add_query_arg( 'tab', $page->getTabSlug() )
            ];
        }, $this->settings_pages );
    }
    
    /**
     * Retrieve the tab-slug currently active on the settings page
     *
     * @since 0.1
     * @return string
     */
    public function getActiveTab() {
        
        if( isset( $_GET[ 'tab' ] ) ) {
            return $_GET[ 'tab' ];
        }
        $settings_pages = $this->getSettingsTabs();
        if( ! $settings_pages->isEmpty() ) {
            return $settings_pages->first()->getTabSlug();
        }
        
        return '';
    }
    
    /**
     * Retrieve a collection of the pages displayed ass tabs on the settings page
     *
     * @since 0.1
     * @return Collection
     */
    public function getSettingsTabs() {
        return new Collection( $this->settings_pages );
    }
    
    /**
     * Retrieve any messages from the current page
     *
     * @since 0.1
     * @return string
     */
    protected function getMessage() {
        return $this->current_page->getMessage();
    }
}