<?php
/**
 * Model: Represents a menu
 */
namespace EwTools\Models;

use EwTools\Support\Collection;
use EwTools\Support\Fluent;
use WP_Term;

/**
 * Menu
 *
 * @link    http://infomaker.se
 * @package Everyware\Models
 * @since   Everyware\Models\Menu 0.1
 */
class Menu extends Fluent {
    
    /**
     * Contains the defaults for fetching menus via WP
     *
     * @var array
     */
    protected static $defaults = [
        'taxonomy'               => 'nav_menu',
        'orderby'                => 'name',
        'order'                  => 'ASC',
        'hide_empty'             => false,
        'include'                => [],
        'exclude'                => [],
        'exclude_tree'           => [],
        'number'                 => '',
        'offset'                 => '',
        'fields'                 => 'all',
        'name'                   => '',
        'slug'                   => '',
        'hierarchical'           => true,
        'search'                 => '',
        'name__like'             => '',
        'description__like'      => '',
        'pad_counts'             => false,
        'get'                    => '',
        'child_of'               => 0,
        'parent'                 => '',
        'childless'              => false,
        'cache_domain'           => 'core',
        'update_term_meta_cache' => true,
        'meta_query'             => ''
    ];
    
    /**
     * Contains a list of items in the menu
     *
     * @var array
     */
    protected $items = [];
    
    /**
     * Menu constructor.
     *
     * @since 0.1
     *
     * @param array|WP_Term $menu
     */
    public function __construct( $menu = [] ) {
        if ( $menu instanceof WP_Term ) {
            $menu = $menu->to_array();
        }
        $this->fill( $menu );
    }
    
    /**
     * Retrieve menu items
     *
     * @since 0.1
     * @return array
     */
    public function items() {
        if( false !== ( $menu_items = wp_get_nav_menu_items( $this->term_id ) ) ) {
            return new Collection( array_map( function ( $menu_item ) {
                return $menu_item->to_array();
            }, $menu_items) );
        }
    }
    
    /**
     * Render the menu
     *
     * @param $options
     *
     * @since 0.1
     * @return void
     */
    public function render( $options ) {
        wp_nav_menu( array_replace_recursive( $options, [
            'menu' => $this->term_id,
            'echo' => true
        ] ) );
    }
    
    /**
     * Retrieve the menu generated into a html string
     *
     * @param  $options
     *
     * @since 0.1
     * @return false|object|string
     */
    public function generate( $options ) {
        return wp_nav_menu( array_replace_recursive( $options, [
            'menu' => $this->term_id,
            'echo' => false
        ] ) );
    }
    
    /**
     * Retrieves all menus
     *
     * @param array $args Optional. Array of arguments passed on to {@see get_terms()}.
     *                    Default empty array.
     *
     * @since 0.1
     * @return array|Collection
     */
    public static function all( $args = [] ) {
        $menus = wp_get_nav_menus( array_replace_recursive( static::$defaults, $args ) );
        
        if( is_array( $menus ) ) {
            return new Collection( array_map( function ( WP_Term $menu ) {
                return new static($menu);
            }, $menus ) );
        }
        
        return [];
    }
}